@@ -600,6 +600,27 @@ const runTests = (baseopts) => {
       spy.should.have.always.been.calledWith(EV_ADD, testPath);
     });
 
+    it('should detect safe-edit', async () => {
+      const testPath = getFixturePath('change.txt');
+      const safePath = getFixturePath('tmp.txt');
+      await write(testPath, dateNow());
+      const watcher = chokidar_watch(testPath, options);
+      const spy = await aspy(watcher, EV_ALL);
+
+      await delay();
+      await write(safePath, dateNow());
+      await fs_rename(safePath, testPath);
+      await delay(100);
+      await write(safePath, dateNow());
+      await fs_rename(safePath, testPath);
+      await delay(100);
+      await write(safePath, dateNow());
+      await fs_rename(safePath, testPath);
+      await waitFor([spy]);
+      spy.withArgs(EV_CHANGE, testPath).should.have.been.calledThrice;
+    });
+
+
     // PR 682 is failing.
     describe.skip('Skipping gh-682: should detect unlink', () => {
       it('should detect unlink while watching a non-existent second file in another directory', async () => {
@@ -1007,6 +1028,7 @@ const runTests = (baseopts) => {
       const watcher = chokidar_watch(watchPaths, options);
       const spy = await aspy(watcher, EV_ALL);
 
+      await waitFor([spy.withArgs(EV_ADD_DIR)]);
       spy.should.have.been.calledWith(EV_ADD_DIR, getFixturePath('subdir'));
       spy.withArgs(EV_ADD_DIR).should.have.been.calledOnce;
       fs.mkdirSync(deepDir, PERM_ARR);
@@ -1962,6 +1984,31 @@ const runTests = (baseopts) => {
         })();
       });
     });
+    it('should not ignore further events on close with existing watchers', async () => {
+      return new Promise((resolve) => {
+        const watcher1 = chokidar_watch(currentDir);
+        const watcher2 = chokidar_watch(currentDir);
+        // The EV_ADD event should be called on the second watcher even if the first watcher is closed
+        watcher2.on(EV_ADD, () => {
+          watcher2.on(EV_ADD, (path) => {
+            if (path.endsWith('add.txt')) {
+              resolve();
+            }
+          })
+        });
+        (async () => {
+          await waitForWatcher(watcher1);
+          await waitForWatcher(watcher2);
+          // Watcher 1 is closed to ensure events only happen on watcher 2
+          await watcher1.close();
+          // Write a new file into the fixtures to test the EV_ADD event
+          await write(getFixturePath('add.txt'), 'hello');
+          // Ensures EV_ADD is called. Immediately removing the file causes it to be skipped
+          await delay(200);
+          await fs_unlink(getFixturePath('add.txt'));
+        })()
+      })
+    });
     it('should not prevent the process from exiting', async () => {
       const scriptFile = getFixturePath('script.js');
       const scriptContent = `
